/*
 *                        sleventrecord.h  -  description
 *                           -------------------
 *  begin                : lun nov 24 2003
 *  copyright            : (C) 2003 by giuseppe massimo bertani
 *  email                : gmbertani@serlook.sunsite.dk
 *
 *
 *
 *                                                                         
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                   
 *                                                                         
 *  Coding style params: 
 *  <TAB> skips=4
 *  <TAB> are replaced with blanks
 *  
 *   Best viewed with Vim 6.x and included vimrc file ;-)
 *   
 * $Id: sleventrecord.h,v 1.6 2004/12/20 09:16:37 gbertani Exp $
 */

#ifndef SLEVENTRECORD_H
#define SLEVENTRECORD_H

//linux
#include <sys/time.h>

//Qt
#include <qfile.h>
#include <qdir.h>

//KDE
#include <kurl.h>

class SLHexBuffer;


enum EventKind
{
    EV_NONE,        //No events
    EV_RX1,         //Receiving from primary port
    EV_RX2,         //Receiving from secondary port
    EV_TX,          //Sending
    EV_STAT1OUT,    //Changing modem status on primary port, output lines (output buffer)
    EV_STAT1IN,     //Changing modem status on primary port, input lines (input buffer)
    EV_STAT2        //Changing modem status on secondary port
}; 

/**
 * Record structure for event.log
 *@author giuseppe massimo bertani
 */

struct SLEvent 
{
    
    /** Progressive event number */
    ulong         progNr;
    
    /** timestamp of this event */
    long          tvSec;
    /** */
    long          tvUSec;

    /** kind of event */
    uchar         kind;

    /** offset to the related data log file, if needed */
    long          offset;
    
    /** size of the data chunk received or sent */
    long          chunkSize;
    
    /** modem line status */
    uint          mStatus;

};


/**
 * Event.log file management
 *
 *@author giuseppe massimo bertani
 */

class SLEventRecord 
{
public: 

    /**
     * Tries first to open the specified file. If exists,
     * it will be opened in append mode and the nextEvId is
     * set accordingly, otherwise the file is created
     * brand new and nextEvId set to zero.
     *
     * The content of the file is loaded in a CHexBuffer 
     * for faster access and written down to the disk only
     * when saving the session, in the same way the data buffers do
     */ 
    SLEventRecord( QString& name );

    /**
     * Flushes the event buffer to the disk 
     * and closes the file
     */ 
	  ~SLEventRecord();

    /**
     * Appends a new event of the specified kind
     */ 
    void addEvent( EventKind k, long off, long size, uint modem );
    
    /**
     * returns the event record given
     * returns true if found, false otherwise
     */
    bool getEvent( ulong prog, SLEvent& ev );

    /**
     * Get the event happened with the given timestamp 
     * 
     * Since the system time on PCs does not reach the
     * mS precision, might be that some consecutive events 
     * could have the same timestamp, in this case the
     * progrId makes the difference.
     * 
     * If Id is zero, the event returned is the first record found 
     * with the given timestamp, if Id is 1 the event returned
     * is the second, etc.
     */
    
    int getEvent( const timeval& ts, uint id, SLEvent& ev );

    /**
     * Returns the progressive number of event that
     * generated the receiving/sending of the data
     * at offset off (warning: it is an offset referred to a
     * data buffer, not to the event buffer!). 
     * 
     */
    ulong getEventProg( EventKind k, long off );
    
    /** number of SLEvent records present 
     * in the buffer */
    ulong getTotalEvents() const;

    /** */
    void getEventTimeval( const SLEvent& ev, timeval& tv ) const;
    
    /** */
    void setEventTimeval( SLEvent& ev, const timeval& tv ) const;
    
    /**
     *
     */
    const QString& getFileName()
    {
        return fileName;
    }   

    /**
     * flushes the buffer
     */
    void clear();
   
    /**
     *
     */
    bool saveBuffer( const QString& newName );
    
    /**
     *
     */
    bool loadBuffer( const QString& name );
    
protected:


    /**
     * returns the difference, as uS, between two times 
     * result is positive if a > b, negative if
     * b > a and 0 if both times are equal
     *
     */
    long timeDiff( const timeval& a, const timeval& b );

    
    /** Last event read from the buffer */
    SLEvent evRead;

    /** Last event appended to the buffer */
    SLEvent evWrite;
   

    /** event buffer file name url */
    QString fileName; 
   
    /** event buffer */
    SLHexBuffer* evBuf; 
};


#endif
